from rest_framework import serializers

from django.contrib.auth import get_user_model
from django.core.files.images import get_image_dimensions
from django.template.defaultfilters import filesizeformat

from .conf import settings
from .models import Avatar


User = get_user_model()


class AvatarSerializer(serializers.ModelSerializer):
    class Meta:
        model = Avatar
        fields = ['id', 'avatar']

    def validate_avatar(self, avatar):
        if avatar.size > settings.AVATAR_MAX_SIZE:
            raise serializers.ValidationError(
                'Your file is too big ({}), the maximum allowed size is {}'.format(
                    filesizeformat(avatar.size),
                    filesizeformat(settings.AVATAR_MAX_SIZE)
                )
            )

        width, height = get_image_dimensions(avatar)

        if width < 200 or height < 200:
            raise serializers.ValidationError(
                'Logo must be at least 200px x 200px'
            )

        return avatar



# Generated by Django 3.1.5 on 2021-02-19 18:37

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import rest_avatar.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Avatar',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_primary', models.BooleanField(default=False)),
                ('avatar', models.ImageField(upload_to=rest_avatar.models.avatar_file_path)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='avatars', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.db import models
from django.contrib.auth import get_user_model


User = get_user_model()


def avatar_file_path(instance, filename):
    return 'avatars/{}/{}'.format(instance.user.id, filename)


class Avatar(models.Model):
    user = models.ForeignKey(
        User, related_name='avatars', on_delete=models.CASCADE
    )
    is_primary = models.BooleanField(default=False)
    avatar = models.ImageField(upload_to=avatar_file_path)

    def set_primary(self):
        primary_avatars = self.user.avatars.filter(is_primary=True)
        primary_avatars = primary_avatars.exclude(pk=self.pk)

        primary_avatars.update(is_primary=False)
        self.is_primary = True
        self.save()



from django.conf import settings

from .models import Avatar


def get_avatar_url(request, user):
    try:
        url = user.avatars.get(is_primary=True).avatar.url
    except Avatar.DoesNotExist:
        # TODO make the image a setting
        url = '{}images/player.png'.format(settings.STATIC_URL)

    return request.build_absolute_uri(url)



from rest_framework import routers

from .views import AvatarViewSet


app_name = 'rest_avatar'

router = routers.DefaultRouter()
router.register('', AvatarViewSet, basename='avatar')


urlpatterns = router.urls



from rest_framework import permissions, status, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from .serializers import AvatarSerializer


class AvatarViewSet(viewsets.ModelViewSet):
    serializer_class = AvatarSerializer
    permission_classes = [permissions.IsAuthenticated]

    def get_queryset(self):
        return self.request.user.avatars.all()

    def perform_create(self, serializer):
        avatar = serializer.save(user=self.request.user)
        avatar.set_primary()

    @action(['patch'], detail=True)
    def set_primary(self, request, *args, **kwargs):
        self.get_object().set_primary()
        return Response(status=status.HTTP_204_NO_CONTENT)
